% Prolog re-implementation of k_ph_n.lwb

k_ph_n(N,File) :-
	tell(File),
	write('begin'),nl,
	k_ph_n(N), nl,
	write('end'),nl,
	told.

k_ph_n(N) :-
	left(N,F1),
	right(N,F2),
	F = neg(implies(dia(r1,F1),dia(r1,F2))),
	printHyLo(F),
	!.

left(N,F) :-
	NP1 is N+1,
	loopLeft1(1,NP1,[],F).

loopLeft1(NP1,NP1,C,F) :-
	N is NP1 - 1,
	loopLeft2(1,N,NP1,[],F1),
	append(C,[F1],C1),
	list2conj(C1,F).
loopLeft1(I,NP1,C,F) :-
	I < NP1,
	N is NP1 - 1,
	loopLeft2(1,N,I,[],F1),
	append(C,[F1],C1),
	IP1 is I+1,
	loopLeft1(IP1,NP1,C1,F).

loopLeft2(N,N,I,D1,F) :-
	l(I,N,F1),
	append(D1,[F1],D2),
	list2disj(D2,F).
loopLeft2(J,N,I,D1,F) :-
	J < N,
	l(I,J,F1),
	append(D1,[F1],D2),
	JP1 is J+1,
	loopLeft2(JP1,N,I,D2,F).

right(N,F) :-
	findall(F1,loopRight(N,F1),List),
	F = or(List).

loopRight(N,F) :-
	NP1 is N + 1,
	between(1,N,J),
	between(1,NP1,I1),
	I1P1 is I1+1,
	between(I1P1,NP1,I2),
	l2(N,I1,J,F1),
	l2(N,I2,J,F2),
	F = and([F1,F2]).

l2(N,I,J,F) :-
	l(I,J,F1),
	K is div((2 * N),3) + 1,
	( (I == J,
	   I == K) ->
	    F = neg(F1)
	;
	    F = F1
	).

l(I,J,F) :-
	K is (I * 1000) + J,
	p(K,F1),
	((I < J) ->
	    F = box(r1,F1)
	;
	    F = F1
	).

p(K,F) :-
	number_chars(K,KChars),
	atom_chars(F,['p'|KChars]),
	!.
	    
list2conj([],true).
list2conj([F1],F1).
list2conj([F1|FL],and([F1|FL])).

list2disj([],false).
list2disj([F1],F1).
list2disj([F1|FL],or([F1|FL])).


printLWB(not(F1)) :-
        write('(~ '), printLWB(F1), write(')'), !.
printLWB(neg(F1)) :-
        write('(~ '), printLWB(F1), write(')'), !.
printLWB(or(F1,F2)) :-
        write('('), printLWB(F1), write(' v '), printLWB(F2), write(')'), !.
printLWB(or([F|FL])) :-
        write('('), printListLWB([F|FL],' v '), write(')'), !.
printLWB(and(F1,F2)) :-
        write('('), printLWB(F1), write(' & '), printLWB(F2), write(')'), !.
printLWB(and([F|FL])) :-
        write('('), printListLWB([F|FL],' & '), write(')'), !.
printLWB(implies(F1,F2)) :-
        write('('), printLWB(F1), write(' -> '), printLWB(F2), write(')'), !.
printLWB(equiv(F1,F2)) :-
        write('('), printLWB(F1), write(' <-> '), printLWB(F2), write(')'), !.
printLWB(box(_R,F1)) :-
        write('(box '), printLWB(F1), write(')'), !.
printLWB(dia(_R,F1)) :-
        write('(dia '), printLWB(F1), write(')'), !.
printLWB(F1) :-
        atomic(F1),
        atom_chars(F1,[v|N]),
        atom_chars(F2,[p|N]),
        write(F2),
        !.
printLWB(S) :-
        write_term(S,[quoted(false),ignore_ops(false),portrayed(true)]).

printListLWB([],_) :- !.
printListLWB([F1],_) :-
	printLWB(F1).
printListLWB([F1,F2|FL],Op) :-
	printLWB(F1),
	write(Op),
	printListLWB([F2|FL],Op).

printHyLo(not(F1)) :-
        write('(~ '), printHyLo(F1), write(')'), !.
printHyLo(neg(F1)) :-
        write('(~ '), printHyLo(F1), write(')'), !.
printHyLo(or(F1,F2)) :-
        write('('), printHyLo(F1), write(' | '), printHyLo(F2), write(')'), !.
printHyLo(or([F|FL])) :-
        write('('), printListHyLo([F|FL],' | '), write(')'), !.
printHyLo(and(F1,F2)) :-
        write('('), printHyLo(F1), write(' & '), printHyLo(F2), write(')'), !.
printHyLo(and([F|FL])) :-
        write('('), printListHyLo([F|FL],' & '), write(')'), !.
printHyLo(implies(F1,F2)) :-
        write('('), printHyLo(F1), write(' -> '), printHyLo(F2), write(')'), !.
printHyLo(equiv(F1,F2)) :-
        write('('), printHyLo(F1), write(' <-> '), printHyLo(F2), write(')'), !.
printHyLo(box(R,F1)) :-
        write('['), write(R), write('] ('), printHyLo(F1), write(')'), !.
printHyLo(dia(R,F1)) :-
        write('<'), write(R), write('> ('), printHyLo(F1), write(')'), !.
printHyLo(F1) :-
        atomic(F1),
        atom_chars(F1,[v|N]),
        atom_chars(F2,[p|N]),
        write(F2),
        !.
printHyLo(S) :-
        write_term(S,[quoted(false),ignore_ops(false),portrayed(true)]).

printListHyLo([],_) :- !.
printListHyLo([F1],_) :-
	printHyLo(F1).
printListHyLo([F1,F2|FL],Op) :-
	printHyLo(F1),
	write(Op),
	printListHyLo([F2|FL],Op).

:- k_ph_n(2,'k_ph_n.0002.intohylo').
:- k_ph_n(4,'k_ph_n.0004.intohylo').
:- k_ph_n(6,'k_ph_n.0006.intohylo').
:- k_ph_n(8,'k_ph_n.0008.intohylo').
:- k_ph_n(10,'k_ph_n.0010.intohylo').
:- k_ph_n(12,'k_ph_n.0012.intohylo').
:- k_ph_n(14,'k_ph_n.0014.intohylo').
:- k_ph_n(16,'k_ph_n.0016.intohylo').
:- k_ph_n(18,'k_ph_n.0018.intohylo').
:- k_ph_n(20,'k_ph_n.0020.intohylo').
:- k_ph_n(25,'k_ph_n.0025.intohylo').
:- k_ph_n(30,'k_ph_n.0030.intohylo').
:- k_ph_n(35,'k_ph_n.0035.intohylo').
:- k_ph_n(40,'k_ph_n.0040.intohylo').
:- k_ph_n(45,'k_ph_n.0045.intohylo').
:- k_ph_n(50,'k_ph_n.0050.intohylo').
:- k_ph_n(55,'k_ph_n.0055.intohylo').
:- k_ph_n(60,'k_ph_n.0060.intohylo').
:- k_ph_n(65,'k_ph_n.0065.intohylo').
:- k_ph_n(70,'k_ph_n.0070.intohylo').
:- k_ph_n(75,'k_ph_n.0075.intohylo').
:- k_ph_n(80,'k_ph_n.0080.intohylo').
:- k_ph_n(85,'k_ph_n.0085.intohylo').
:- k_ph_n(90,'k_ph_n.0090.intohylo').
:- k_ph_n(95,'k_ph_n.0095.intohylo').
:- k_ph_n(100,'k_ph_n.0100.intohylo').
:- k_ph_n(105,'k_ph_n.0105.intohylo').
:- k_ph_n(110,'k_ph_n.0110.intohylo').
:- k_ph_n(115,'k_ph_n.0115.intohylo').
:- k_ph_n(120,'k_ph_n.0120.intohylo').
:- k_ph_n(125,'k_ph_n.0125.intohylo').
:- k_ph_n(130,'k_ph_n.0130.intohylo').
:- k_ph_n(135,'k_ph_n.0135.intohylo').
:- k_ph_n(140,'k_ph_n.0140.intohylo').
:- k_ph_n(145,'k_ph_n.0145.intohylo').
:- k_ph_n(150,'k_ph_n.0150.intohylo').
:- k_ph_n(155,'k_ph_n.0155.intohylo').
:- k_ph_n(160,'k_ph_n.0160.intohylo').
:- k_ph_n(165,'k_ph_n.0165.intohylo').
:- k_ph_n(170,'k_ph_n.0170.intohylo').
:- k_ph_n(175,'k_ph_n.0175.intohylo').
:- k_ph_n(180,'k_ph_n.0180.intohylo').
:- k_ph_n(185,'k_ph_n.0185.intohylo').
:- k_ph_n(190,'k_ph_n.0190.intohylo').
:- k_ph_n(195,'k_ph_n.0195.intohylo').
:- k_ph_n(200,'k_ph_n.0200.intohylo').
:- k_ph_n(205,'k_ph_n.0205.intohylo').
:- k_ph_n(210,'k_ph_n.0210.intohylo').
:- k_ph_n(215,'k_ph_n.0215.intohylo').
:- k_ph_n(220,'k_ph_n.0220.intohylo').
:- k_ph_n(225,'k_ph_n.0225.intohylo').
:- k_ph_n(230,'k_ph_n.0230.intohylo').
:- k_ph_n(235,'k_ph_n.0235.intohylo').
:- k_ph_n(240,'k_ph_n.0240.intohylo').
:- k_ph_n(245,'k_ph_n.0245.intohylo').
:- k_ph_n(250,'k_ph_n.0250.intohylo').
